<?php

require_once(dirname(__FILE__) . '/open_util.class.php');

/**
 * 开放银行平台统一接口类<br />
 *
 * <b>[注意]</b> 实例化该类时需要传入config参数，示例参数请见open_config.php文件<br />
 * 该SDK中代码仅供学习和参考，开发者可以自己根据接口文档进行编码实现，而不使用该SDK中的实现<br />
 * 各字段长度、类型等请参考具体接口文档中的定义，SDK不检查这些长度，而直接发送给服务端<br />
 * [重要]各传入参数SDK都不作任何检查、过滤，请务必在传入前进行安全检查或过滤，否则可能会导致安全问题
 *
 * @author xiezz, zengminghua
 */
Class OpenSDK
{

    // 配置参数，示例中在open_config.php中定义
    private $open_config;

    // 构造函数
    public function __construct($open_config)
    {
        $this->open_config = $open_config;
    }

    // 兼容旧版本PHP的构造函数
    public function OpenSDK($open_config)
    {
        $this->__construct($open_config);
    }

    // 开放银行平台生产环境地址，无需修改
    const PROD_API = 'https://open.cibfintech.com';

    // 开放银行平台测试环境地址，可根据需要修改
    const DEV_API = 'https://open.test.cibfintech.com';

    /**
     * 生成签名字符串
     * @param string $keyid 秘钥标识ID
     * @param string $uri 请求资源
     * @param string $method HTTP方法
     * @param string $timestamp 时间戳
     * @param array $param_array 参数列表
     * @param string $privateKey 请求报文签名私钥
     * @return string MAC字符串
     */
    public static function Signature($keyid, $uri, $method, $timestamp, $param_array, $privateKey)
    {
        $signstr = '';

        if ($param_array != null) {
            ksort($param_array);
            reset($param_array);

            foreach ($param_array as $k => $v) {
                $signstr .= "{$k}={$v}&";
            }
        }

        $signstr = $keyid . '&' . $timestamp . '&' . $method . '&' . $uri . '&' . $signstr;
        $signstr = substr($signstr, 0, strlen($signstr) - 1);
        $priv_key = openssl_pkey_get_private($privateKey);
        if ($priv_key && openssl_sign($signstr, $sign, $priv_key, 'sha256WithRSAEncryption')) {
            openssl_free_key($priv_key);
            return base64_encode($sign);
        } else {
            @openssl_free_key($priv_key);
            return 'SIGNATURE_RSA_PRIVATEKEY_ERROR';
        }
    }


    /////////////////////////////////////////////////////////////////////////////////////////////


    /**
     * 内部数据下传调用入口
     * @param string $uri 请求资料地址
     * @param string $method HTTP方法
     * @param array $header_param_array 头部参数
     * @param array $url_param_array URL参数
     * @param array $body_param_array body参数，GET方法时请勿送
     * @param string $save_file_name 结果保存至文件时的全路径
     * @param boolean $use_multipart 是否使用multi-part格式
     * @return string json格式结果，返回结果包含字段请参看接口文档
     */
    public function txn($uri, $method, $header_param_array, $url_param_array, $body_param_array, $save_file_name = null)
    {

        if ($uri == null || !is_string($uri)
            || $method == null || !(strcmp("GET", $method) == 0 || strcmp("POST", $method) == 0)
            || $header_param_array !== null && !is_array($header_param_array)
            || $url_param_array !== null && !is_array($url_param_array)
            || $body_param_array !== null && !is_array($body_param_array)
        ) {
            return INVALID_PARAM_RESULT;
        }

        if (strcmp('/', substr($uri, 0, 1)) != 0)
            return INVALID_PARAM_RESULT;

        $sign_param_array = array();

        if (is_array($header_param_array)) $sign_param_array = $sign_param_array + $header_param_array;
        if (is_array($url_param_array)) $sign_param_array = $sign_param_array + $url_param_array;
        if (is_array($body_param_array) && strcmp("POST", $method) == 0) $sign_param_array = $sign_param_array + $body_param_array;

        $now = OpenUtil::getDateTime();
        $sign_result = self::Signature($this->open_config['keyid'], $uri, $method, $now, $sign_param_array, $this->open_config['privateKey']);
        if (!is_array($header_param_array)) $header_param_array = array();
        $header_param_array['Authorization'] = 'Basic ' . base64_encode($this->open_config['keyid'] . '_' . $now . ':' . $sign_result);

        $response = OpenUtil::getHttpResponse(($this->open_config['isDevEnv'] === false ? OpenSDk::PROD_API : OpenSDk::DEV_API) . $uri, $method, $url_param_array, $body_param_array, $header_param_array, false, $this->open_config['proxy_ip'], $this->open_config['proxy_port'], $save_file_name, false);

        if (!$response || !is_string($response))
            return TXN_ERROR_RESULT;
        else
            return $response;
    }

    /**
     * 统一接口调用入口
     * @param string $uri 请求资料地址
     * @param string $method HTTP方法
     * @param array $header_param_array 头部参数
     * @param array $url_param_array URL参数
     * @param array $body_param_array body参数，GET方法时请勿送
     * @return string json格式结果，返回结果包含字段请参看接口文档
     */
    public function exec($uri, $method, $header_param_array, $url_param_array, $body_param_array)
    {
        return $this->txn($uri, $method, $header_param_array, $url_param_array, $body_param_array, null);
    }

    /**
     * 文件下载接口
     * @param $file_id 待下载文件fileid
     * @param $save_file_name 保存至本地的全路径
     * @return string 调用结果，json格式
     */
    public function download_file($file_id, $save_file_name)
    {
        $url_param_array = array();
        $url_param_array['fileId'] = $file_id;
        return $this->txn('/api/open/downloadFile', 'GET', null, $url_param_array, null, $save_file_name);
    }

    /**
     * 文件上传接口
     * @param $upload_file_name 待上传的文件全路径
     * @return string 调用结果，json格式
     */
    public function upload_file($upload_file_name)
    {
        if (!file_exists($upload_file_name)) return FILE_ERROR_RESULT;
        $upload_file_name = realpath($upload_file_name);

        $sha1 = sha1_file($upload_file_name);
        $uri = '/api/open/uploadFile';

        $param_array = array();
        $param_array['fileHashValue'] = $sha1;

        $now = OpenUtil::getDateTime();
        $sign_result = self::Signature($this->open_config['keyid'], $uri, 'POST', $now, $param_array, $this->open_config['privateKey']);

        $header_param_array['Authorization'] = 'Basic ' . base64_encode($this->open_config['keyid'] . '_' . $now . ':' . $sign_result);

        if (class_exists('\CURLFile')) {
            $param_array['file'] = new \CURLFile($upload_file_name);
        } else {
            $param_array['file'] = '@' . $upload_file_name;
        }
        $response = OpenUtil::getHttpResponse(($this->open_config['isDevEnv'] === false ? OpenSDk::PROD_API : OpenSDk::DEV_API) . $uri, 'POST', null, $param_array, $header_param_array, false, $this->open_config['proxy_ip'], $this->open_config['proxy_port'], null, true);
        return $response;
    }
}
